unit PHSelect;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Jpeg;

type
  TRecordForm = class(TForm)
    LabelName: TLabel;
    EditName: TEdit;
    LabelYear: TLabel;
    EditYear: TEdit;
    LabelRemark: TLabel;
    EditRemark: TEdit;
    CheckBoxDeleted: TCheckBox;
    ClearBtn: TButton;
    LoadBtn: TButton;
    SaveBtn: TButton;
    OKBtn: TButton;
    CancelBtn: TButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    PanelImage: TPanel;
    Image: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LoadBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    ImageChanged: Boolean;
    JPEGImage: TJPEGImage;
    procedure ShowImage;
    procedure ClearImage;
  end;

var
  RecordForm: TRecordForm;

implementation

{$R *.DFM}

procedure TRecordForm.FormCreate(Sender: TObject);
begin
  ImageChanged:=FALSE;
  JPEGImage:=TJPEGImage.Create;
end;

procedure TRecordForm.FormDestroy(Sender: TObject);
begin
  JPEGImage.Free;
end;

procedure TRecordForm.LoadBtnClick(Sender: TObject);
var PictName: String;
begin
  ImageChanged:=TRUE;
  if OpenDialog.Execute then
     begin
       PictName:=OpenDialog.FileName;
       try
          JPEGImage.LoadFromFile(PictName);
          ShowImage;
       except
          MessageDlg('Error reading file '+PictName,
            mtError,[mbOK],0);
          JPEGImage.Free;
          JPEGImage:=TJpegImage.Create;
          ClearImage;
       end;
     end;
end;

procedure TRecordForm.SaveBtnClick(Sender: TObject);
var PictName: String;
    Allow: Boolean;
begin
  if Image.Picture.Graphic=NIL
     then exit;
  if SaveDialog.Execute then
     begin
       PictName:=SaveDialog.FileName;
       Allow:=TRUE;
       if FileExists(PictName) then
          if MessageDlg('File '+PictName+' already exists. Overwrite?',
               mtWarning, [mbYes, mbNo], 0)<>mrYes
               then Allow:=FALSE;
       if Allow
          then JpegImage.SaveToFile(PictName);
     end;
end;

procedure TRecordForm.ClearBtnClick(Sender: TObject);
begin
  ImageChanged:=TRUE;
  ClearImage;
end;

procedure TRecordForm.ShowImage;
var W1, H1, W2, H2: Integer;
begin
  Image.Picture.Assign(JPEGImage);
  W1:=JPEGImage.Width;
  H1:=JPEGImage.Height;
  if (W1>200) OR (H1>200) then
     begin
       Image.Stretch:=TRUE;
       if W1>H1 then
          begin
            W2:=200;
            H2:=Round(H1*W2/W1);
          end
       else
          begin
            H2:=200;
            W2:=Round(W1*H2/H1);
          end;
       Image.Left:=1 + (200-W2) div 2;
       Image.Top:=1 + (200-H2) div 2;
       Image.Width:=W2;
       Image.Height:=H2;
     end
  else
     begin
       Image.Stretch:=FALSE;
       Image.Left:=1;
       Image.Top:=1;
       Image.Width:=200;
       Image.Height:=200;
     end;
end;

procedure TRecordForm.ClearImage;
begin
  JPEGImage.Free;
  JPEGImage:=TJpegImage.Create;
  Image.Picture.Assign(NIL);
end;

end.

