unit ODBFForm;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, ODBF, Grids, ODBFTool, ODBFCprt, ODBFMemo;

const ScaleCoef = 1.2;

type
  TMainForm = class(TForm)
    DBFStructure: TDBFStructure;
    MainMenu: TMainMenu;
    NFile: TMenuItem;
    NNew: TMenuItem;
    NOpen: TMenuItem;
    N1: TMenuItem;
    NExit: TMenuItem;
    NHelp: TMenuItem;
    NAbout: TMenuItem;
    SaveDialog: TSaveDialog;
    StringGrid: TStringGrid;
    OpenDialog: TOpenDialog;
    NViewStructure: TMenuItem;
    NTable: TMenuItem;
    NViewMemo: TMenuItem;
    NEdit: TMenuItem;
    NAddRecord: TMenuItem;
    NDeleteRecord: TMenuItem;
    NChangeMemo: TMenuItem;
    N2: TMenuItem;
    NPackRecords: TMenuItem;
    NPackMemo: TMenuItem;
    NUndeleteRecord: TMenuItem;
    NLoadMemo: TMenuItem;
    NLoadBlob: TMenuItem;
    NSaveMemo: TMenuItem;
    NSaveBlob: TMenuItem;
    DBFTable: TDBFTable;
    NBlobMemo: TMenuItem;
    N3: TMenuItem;
    NRestructure: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure NNewClick(Sender: TObject);
    procedure NOpenClick(Sender: TObject);
    procedure NViewStructureClick(Sender: TObject);
    procedure NExitClick(Sender: TObject);
    procedure NAboutClick(Sender: TObject);
    procedure NViewMemoClick(Sender: TObject);
    procedure NAddRecordClick(Sender: TObject);
    procedure NDeleteRecordClick(Sender: TObject);
    procedure NUndeleteRecordClick(Sender: TObject);
    procedure NChangeMemoClick(Sender: TObject);
    procedure NPackRecordsClick(Sender: TObject);
    procedure NPackMemoClick(Sender: TObject);
    procedure UpdateCurrentRecord;
    procedure StringGridSetEditText(Sender: TObject; ACol, ARow: Longint;
      const Value: string);
    procedure StringGridSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure NLoadMemoClick(Sender: TObject);
    procedure NLoadBlobClick(Sender: TObject);
    procedure NSaveMemoClick(Sender: TObject);
    procedure NSaveBlobClick(Sender: TObject);
    procedure NRestructureClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    CurRow: Longint;
    Editing: Boolean;
    procedure AdjustColWidths;
    procedure ShowTable;
    procedure LoadDataFromFile (DataClass: Integer);   {0 - Memo, 1 - Blob}
    procedure SaveDataToFile (DataClass: Integer);   {0 - Memo, 1 - Blob}
  end;

var
  MainForm: TMainForm;
  ShowError: Boolean;

implementation

{$R *.DFM}

procedure TMainForm.AdjustColWidths;
var I, J: Integer;
begin
  with StringGrid do
    for I:=0 to ColCount-1 do
      begin
        ColWidths[I]:=DefaultColWidth;
        for J:=0 to RowCount-1 do
          if Canvas.TextWidth(Cells[I,J])>ColWidths[I]
             then ColWidths[I]:=Canvas.TextWidth(Cells[I,J]);
        ColWidths[I]:=Round(ColWidths[I]*ScaleCoef);
      end;
end;

procedure TMainForm.ShowTable;
var ADBFField: TDBFField;
    Index: Longint;
    I: Integer;
    S: String;
    N: Real;
    ADate: TDateTime;
    L: Boolean;
begin
  with StringGrid do
    begin
      ColCount:=1+DBFTable.FieldCount;
      RowCount:=1+DBFTable.RecordCount;
      Cells[0,0]:='<Del>';
      for I:=0 to DBFTable.FieldCount-1 do
        begin
          ADBFField:=DBFTable.Fields[I];
          Cells[I+1,0]:=ADBFField.FieldName;
        end;
      for Index:=1 to DBFTable.RecordCount do
        begin
          try
             DBFTable.RecNo:=Index;
          except
             on E: Exception do
                begin
                  RowCount:=Index;
                  if ShowError then
                     if MessageDlg(Format('Exception occurs: <%s>. Show this message again?',[E.Message]),
                        mtError,[mbYes,mbNo],0)=mrNo
                        then ShowError:=FALSE;
                  break;
                  {AdjustColWidths;
                  Caption:='DBF Table Viewer';
                  if DBFTable.FileName<>''
                     then Caption:=Caption+' - ['+DBFTable.FileName+']';
                  raise;}
                end;
          end;
          if DBFTable.Deleted
             then Cells[0,Index]:='*'
             else Cells[0,Index]:=' ';
          for I:=0 to DBFTable.FieldCount-1 do
            begin
              ADBFField:=DBFTable.Fields[I];
              case ADBFField.FieldType of
                'C': begin
                       S:=ADBFField.Text;
                       {StrPCopy(Buffer,S);
                       OemToAnsi(Buffer,Buffer);
                       S:=StrPas(Buffer);}
                     end;
                'N': begin
                       if ADBFField.Defined then
                          begin
                            N:=ADBFField.AsFloat;
                            Str(N:ADBFField.FieldLength:ADBFField.FieldDecimals,S);
                          end
                       else S:='';
                     end;
                'D': begin
                       if ADBFField.Defined then
                          begin
                            ADate:=ADBFField.AsDateTime;
                            S:=DateToStr(ADate);
                          end
                       else S:='';
                     end;
                'L': begin
                       if ADBFField.Defined then
                          begin
                            L:=ADBFField.AsBoolean;
                            case L of
                              TRUE: S:='TRUE';
                              FALSE: S:='FALSE';
                            end;
                          end
                       else S:='';
                     end;
                'M': begin
                       if ADBFField.Defined
                          then S:='<MEMO>'
                          else S:='';
                     end;
              end;
              Cells[I+1,Index]:=S;
            end;
        end;
    end;
  AdjustColWidths;
  Caption:='DBF Table Editor';
  if DBFTable.FileName<>''
     then Caption:=Caption+' - ['+DBFTable.FileName+']';
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  CurRow:=0;
  Editing:=FALSE;
  NEdit.Enabled:=FALSE;
  NTable.Enabled:=FALSE;
  NBlobMemo.Enabled:=FALSE;
end;

procedure TMainForm.NNewClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  DBFStructure.Clear;
  SaveDialog.Filter:='DBase tables (*.dbf)|*.dbf|All files (*.*)|*.*';
  if DBFStructure.DefineFields=mrOK then
     if SaveDialog.Execute then
        try
           DBFTable.CreateTable(SaveDialog.FileName,DBFStructure);
           DBFTable.AppendBlank;
           ShowTable;
           CurRow:=0;
           Editing:=FALSE;
           NEdit.Enabled:=TRUE;
           NTable.Enabled:=TRUE;
           NBlobMemo.Enabled:=TRUE;
        except
           NEdit.Enabled:=FALSE;
           NTable.Enabled:=FALSE;
           NBlobMemo.Enabled:=FALSE;
           raise;
        end;
end;

procedure TMainForm.NOpenClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  OpenDialog.Filter:='DBase tables (*.dbf)|*.dbf|All files (*.*)|*.*';
  if OpenDialog.Execute then
     try
        DBFTable.Use(OpenDialog.FileName);
        ShowError:=TRUE;
        ShowTable;
        CurRow:=0;
        Editing:=FALSE;
        NEdit.Enabled:=TRUE;
        NTable.Enabled:=TRUE;
        NBlobMemo.Enabled:=TRUE;
     except
        NEdit.Enabled:=FALSE;
        NTable.Enabled:=FALSE;
        NBlobMemo.Enabled:=FALSE;
        raise;
     end;
end;

procedure TMainForm.NViewStructureClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFStructure.FieldList:=DBFTable.FieldDefs.FieldList;
  if DBFStructure.DefineFields<>mrCancel
     then MessageDlg('Changes will have no effect',mtInformation,[mbOK],0);
end;

procedure TMainForm.NExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.NAboutClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  AboutBox.ShowModal;
end;

procedure TMainForm.NViewMemoClick(Sender: TObject);
var ADBFField: TDBFField;
    Buffer: Array [0..255] of Char;
    FName: String;
    IH: Word;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFTable.RecNo:=StringGrid.Row;
  ADBFField:=DBFTable.Fields[StringGrid.Col-1];
  if (ADBFField.FieldType='M') then
     begin
       GetUniqueName('',Buffer);
       FName:=StrPas(Buffer);
       if ADBFField.GetMemoToFile(FName) then
          begin
            StrPCopy(Buffer,'Notepad.exe '+FName);
            IH:=WinExec(Buffer,sw_Normal);
            if IH<32
               then MessageDlg('Failed exec of Notepad',mtError,[mbOK],0);
            DeleteFile(FName);
          end
       else MessageDlg('Field value is not defined',mtWarning,[mbOK],0);
     end
  else MessageDlg('Not a memo field',mtWarning,[mbOK],0);
end;

procedure TMainForm.NAddRecordClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  DBFTable.AppendBlank;
  ShowTable;
end;

procedure TMainForm.NDeleteRecordClick(Sender: TObject);
var J: Longint;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  J:=StringGrid.Row;
  if J<=0 then exit;
  DBFTable.RecNo:=J;
  if Not DBFTable.Deleted then
     begin
       DBFTable.Delete;
       DBFTable.ReplaceRecord;   {Not necessary when AutoUpdate=TRUE}
     end;
  ShowTable;
end;

procedure TMainForm.NUndeleteRecordClick(Sender: TObject);
var J: Longint;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  J:=StringGrid.Row;
  if J<=0 then exit;
  DBFTable.RecNo:=J;
  if DBFTable.Deleted then
     begin
       DBFTable.Undelete;
       DBFTable.ReplaceRecord;   {Not necessary when AutoUpdate=TRUE}
     end;
  ShowTable;
end;

procedure TMainForm.NChangeMemoClick(Sender: TObject);
var ADBFField: TDBFField;
    Buffer: Array [0..255] of Char;
    FName: String;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFTable.RecNo:=StringGrid.Row;
  ADBFField:=DBFTable.Fields[StringGrid.Col-1];
  if (ADBFField.FieldType='M') then
     begin
       GetUniqueName('',Buffer);
       FName:=StrPas(Buffer);
       ADBFField.GetMemoToFile(FName);
       MemoDlg:=TMemoDlg.Create(NIL);
       try
          if FileExists(FName)
             then MemoDlg.Memo.Lines.LoadFromFile(FName);
          if MemoDlg.ShowModal=mrOK then
             begin
               if (MemoDlg.Memo.Lines.Count=0) OR
                  ((MemoDlg.Memo.Lines.Count=1) AND
                  (MemoDlg.Memo.Lines[0]=''))
                  then ADBFField.ClearValue
                  else
                    begin
                      MemoDlg.Memo.Lines.SaveToFile(FName);
                      ADBFField.SetMemoFromFile(FName);
                    end;
               DBFTable.ReplaceRecord;      {Not necessary when AutoUpdate=TRUE}
               ShowTable;
             end;
       finally
          MemoDlg.Free;
          if FileExists(FName) then DeleteFile(FName);
       end;
     end
  else MessageDlg('Not a memo field',mtWarning,[mbOK],0);
end;

procedure TMainForm.NPackRecordsClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  DBFTable.Pack;
  ShowTable;
end;

procedure TMainForm.NPackMemoClick(Sender: TObject);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
  DBFTable.PackMemo;
  ShowTable;
end;

procedure TMainForm.UpdateCurrentRecord;
var ACol: Longint;
    DBFField: TDBFField;
    N: Real;
    Code: Integer;
    ADate: TDateTime;
begin
  if (CurRow<=0) OR (CurRow>DBFTable.RecordCount) then exit;
  DBFTable.Go(CurRow);
  for ACol:=1 to DBFTable.FieldCount do
    begin
      DBFField:=DBFTable.Fields[ACol-1];
      if DBFField=NIL then exit;
      case DBFField.FieldType of
        'C': DBFField.AsString:=StringGrid.Cells[ACol,CurRow];
        'N': begin
               if StringGrid.Cells[ACol,CurRow]='' then
                  DBFField.ClearValue
               else
                  begin
                    Val(StringGrid.Cells[ACol,CurRow],N,Code);
                    if Code<>0 then
                       begin
                         MessageDlg(DBFField.FieldName+
                           ': New value is not a valid number',mtWarning,[mbOK],0);
                         ShowTable;
                         exit;
                       end;
                    DBFField.AsFloat:=N;
                  end;
             end;
        'D': begin
               if StringGrid.Cells[ACol,CurRow]='' then
                  DBFField.ClearValue
               else
                  begin
                    try
                       ADate:=StrToDate(StringGrid.Cells[ACol,CurRow]);
                    except
                       MessageDlg(DBFField.FieldName+
                         'New value is not a valid date',mtWarning,[mbOK],0);
                       ShowTable;
                       exit;
                    end;
                    DBFField.AsDateTime:=ADate;
                  end;
             end;
        'L': begin
               if StringGrid.Cells[ACol,CurRow]=''
                  then DBFField.ClearValue else
               if StringGrid.Cells[ACol,CurRow][1] in ['t','T','y','Y','1']
                  then DBFField.AsBoolean:=TRUE else
               if StringGrid.Cells[ACol,CurRow][1] in ['f','F','n','N','0']
                  then DBFField.AsBoolean:=FALSE else
                  begin
                    MessageDlg(DBFField.FieldName+
                      'New value is not a valid boolean',mtWarning,[mbOK],0);
                    ShowTable;
                    exit;
                  end;
             end;
      end;
    end;
  DBFTable.ReplaceRecord;   {Not necessary when AutoUpdate=TRUE}
  ShowTable;
  Editing:=FALSE;
end;

procedure TMainForm.StringGridSetEditText(Sender: TObject; ACol,
  ARow: Longint; const Value: string);
begin
  if ARow=CurRow then Editing:=TRUE;
end;

procedure TMainForm.StringGridSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
begin
  if (CurRow<>Row) then
     begin
       if (Editing) then
          begin
            if MessageDlg('Do you wish to update the record No '+
               IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
               then UpdateCurrentRecord;
          end
       else ShowTable;
       CurRow:=Row;
       Editing:=FALSE;
       StringGrid.EditorMode:=FALSE;
     end;
  CanSelect:=TRUE;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end;
end;

procedure TMainForm.LoadDataFromFile (DataClass: Integer);
var ADBFField: TDBFField;
    Buffer: Array [0..255] of Char;
    FName: String;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFTable.RecNo:=StringGrid.Row;
  ADBFField:=DBFTable.Fields[StringGrid.Col-1];
  if (ADBFField.FieldType='M') then
     begin
       OpenDialog.Filter:='All files (*.*)|*.*';
       if OpenDialog.Execute then
          begin
            if DataClass=0
               then ADBFField.SetMemoFromFile(OpenDialog.FileName)
               else ADBFField.SetBlobFromFile(OpenDialog.FileName);
            DBFTable.ReplaceRecord;      {Not necessary when AutoUpdate=TRUE}
            ShowTable;
          end;
     end
  else MessageDlg('Not a memo field',mtWarning,[mbOK],0);
end;

procedure TMainForm.NLoadMemoClick(Sender: TObject);
begin
  LoadDataFromFile(0);
end;

procedure TMainForm.NLoadBlobClick(Sender: TObject);
begin
  LoadDataFromFile(1);
end;

procedure TMainForm.SaveDataToFile (DataClass: Integer);
var ADBFField: TDBFField;
    Buffer: Array [0..255] of Char;
    FName: String;
    SaveRes: Boolean;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFTable.RecNo:=StringGrid.Row;
  ADBFField:=DBFTable.Fields[StringGrid.Col-1];
  if (ADBFField.FieldType='M') then
     begin
       SaveDialog.Filter:='All files (*.*)|*.*';
       if SaveDialog.Execute then
          begin
            if DataClass=0
               then SaveRes:=ADBFField.GetMemoToFile(SaveDialog.FileName)
               else SaveRes:=ADBFField.GetBlobToFile(SaveDialog.FileName);
          end;
       if not SaveRes
          then MessageDlg('Field value is not defined',mtWarning,[mbOK],0);
     end
  else MessageDlg('Not a memo field',mtWarning,[mbOK],0);
end;

procedure TMainForm.NSaveMemoClick(Sender: TObject);
begin
  SaveDataToFile(0);
end;

procedure TMainForm.NSaveBlobClick(Sender: TObject);
begin
  SaveDataToFile(1);
end;

procedure TMainForm.NRestructureClick(Sender: TObject);
var Index: Integer;
begin
  if (CurRow>0) AND (Editing) then
     begin
       if MessageDlg('Do you wish to update the record No '+
          IntToStr(CurRow)+'?',mtConfirmation,[mbYes,mbNo],0)=mrYes
          then UpdateCurrentRecord;
     end
  else ShowTable;
  DBFStructure.FieldList:=DBFTable.FieldDefs.FieldList;
  for Index:=0 to DBFStructure.FieldCount-1 do
    DBFStructure.Items[Index].Tag:=Index;
  if DBFStructure.DefineFields=mrOK then
     begin
       if MessageDlg('Do you really want to restructure the table?',
          mtConfirmation,[mbYes,mbNo],0)=mrYes then
          begin
            DBFTable.RestructureTable(DBFStructure);
            ShowTable;
            MessageDlg('Restructuring complete',mtInformation,[mbOK],0);
          end;
     end;
end;

end. 

